@@ -1,5 +1,5 @@
 # testing/requirements.py
-# Copyright (C) 2005-2016 the SQLAlchemy authors and contributors
+# Copyright (C) 2005-2018 the SQLAlchemy authors and contributors
 # <see AUTHORS file>
 #
 # This module is part of SQLAlchemy and is released under
@@ -173,6 +173,19 @@ class SuiteRequirements(Requirements):
         """Target database must support window functions."""
         return exclusions.closed()
 
+    @property
+    def ctes(self):
+        """Target database supports CTEs"""
+
+        return exclusions.closed()
+
+    @property
+    def ctes_on_dml(self):
+        """target database supports CTES which consist of INSERT, UPDATE
+        or DELETE"""
+
+        return exclusions.closed()
+
     @property
     def autoincrement_insert(self):
         """target platform generates new surrogate integer primary key values
@@ -193,6 +206,40 @@ class SuiteRequirements(Requirements):
 
         return exclusions.open()
 
+    @property
+    def group_by_complex_expression(self):
+        """target platform supports SQL expressions in GROUP BY
+
+        e.g.
+
+        SELECT x + y AS somelabel FROM table GROUP BY x + y
+
+        """
+
+        return exclusions.open()
+
+    @property
+    def sane_rowcount(self):
+        return exclusions.skip_if(
+            lambda config: not config.db.dialect.supports_sane_rowcount,
+            "driver doesn't support 'sane' rowcount"
+        )
+
+    @property
+    def sane_multi_rowcount(self):
+        return exclusions.fails_if(
+            lambda config: not config.db.dialect.supports_sane_multi_rowcount,
+            "driver %(driver)s %(doesnt_support)s 'sane' multi row count"
+        )
+
+    @property
+    def sane_rowcount_w_returning(self):
+        return exclusions.fails_if(
+            lambda config:
+                not config.db.dialect.supports_sane_rowcount_returning,
+            "driver doesn't support 'sane' rowcount when returning is on"
+        )
+
     @property
     def empty_inserts(self):
         """target platform supports INSERT with no values, i.e.
@@ -219,6 +266,14 @@ class SuiteRequirements(Requirements):
             "%(database)s %(does_support)s 'returning'"
         )
 
+    @property
+    def tuple_in(self):
+        """Target platform supports the syntax
+        "(x, y) IN ((x1, y1), (x2, y2), ...)"
+        """
+
+        return exclusions.closed()
+
     @property
     def duplicate_names_in_cursor_description(self):
         """target platform supports a SELECT statement that has
@@ -321,6 +376,10 @@ class SuiteRequirements(Requirements):
     def table_reflection(self):
         return exclusions.open()
 
+    @property
+    def comment_reflection(self):
+        return exclusions.closed()
+
     @property
     def view_column_reflection(self):
         """target database must support retrieval of the columns in a view,
@@ -381,6 +440,11 @@ class SuiteRequirements(Requirements):
         """target dialect supports reflection of unique constraints"""
         return exclusions.open()
 
+    @property
+    def check_constraint_reflection(self):
+        """target dialect supports reflection of check constraints"""
+        return exclusions.closed()
+
     @property
     def duplicate_key_raises_integrity_error(self):
         """target dialect raises IntegrityError when reporting an INSERT
@@ -434,6 +498,13 @@ class SuiteRequirements(Requirements):
 
         return exclusions.open()
 
+    @property
+    def timestamp_microseconds(self):
+        """target dialect supports representation of Python
+        datetime.datetime() with microsecond objects but only
+        if TIMESTAMP is used."""
+        return exclusions.closed()
+
     @property
     def datetime_historic(self):
         """target dialect supports representation of Python
@@ -500,6 +571,11 @@ class SuiteRequirements(Requirements):
 
         return exclusions.open()
 
+    @property
+    def autocommit(self):
+        """target dialect supports 'AUTOCOMMIT' as an isolation_level"""
+        return exclusions.closed()
+
     @property
     def json_type(self):
         """target platform implements a native JSON type."""
@@ -539,6 +615,22 @@ class SuiteRequirements(Requirements):
         """
         return exclusions.closed()
 
+    @property
+    def nested_aggregates(self):
+        """target database can select an aggregate from a subquery that's
+        also using an aggregate
+
+        """
+        return exclusions.open()
+
+    @property
+    def recursive_fk_cascade(self):
+        """target database must support ON DELETE CASCADE on a self-referential
+        foreign key
+
+        """
+        return exclusions.open()
+
     @property
     def precision_numerics_retains_significant_digits(self):
         """A precision numeric type will return empty significant digits,
@@ -618,6 +710,11 @@ class SuiteRequirements(Requirements):
         """Target must support UPDATE..FROM syntax"""
         return exclusions.closed()
 
+    @property
+    def delete_from(self):
+        """Target must support DELETE FROM..FROM or DELETE..USING syntax"""
+        return exclusions.closed()
+
     @property
     def update_where_target_in_subquery(self):
         """Target must support UPDATE where the same table is present in a
@@ -666,6 +763,20 @@ class SuiteRequirements(Requirements):
         """
         return exclusions.closed()
 
+    @property
+    def order_by_collation(self):
+        def check(config):
+            try:
+                self.get_order_by_collation(config)
+                return False
+            except NotImplementedError:
+                return True
+
+        return exclusions.skip_if(check)
+
+    def get_order_by_collation(self, config):
+        raise NotImplementedError()
+
     @property
     def unicode_connections(self):
         """Target driver must support non-ASCII characters being passed at
